@@ -77,7 +77,32 @@ struct rngs {
 /* Some semi random spinners to spin,
  * this is 20 bytes of semi random data */
 #define RANDNUMS 5
-static int randnums[RANDNUMS];
+static guint32 randnums[RANDNUMS];
+
+/* stolen from XDM which in turn stole this
+   from the C standard */
+static guint32
+next_rand_15 (guint32 num)
+{
+	num = num * 1103515245 + 12345;
+	return (unsigned int)(num/65536) % 32768;
+}
+
+/* really quite apparently only 31 bits of entropy result (from tests),
+   but oh well */
+static guint32
+next_rand_32 (guint32 num)
+{
+	int i;
+	guint32 ret;
+	guint8 *p = (guint8 *)&ret;
+
+	for (i = 0; i < 4; i++)  {
+		num = next_rand_15 (num);
+		p[i] = (num & 0xff00) >> 8;
+	}
+	return ret;
+}
 
 /* This adds a little bit of entropy to our buffer,
    just in case /dev/random doesn't work out for us
@@ -88,19 +113,30 @@ gdm_random_tick (void)
 {
 	struct timeval tv;
 	struct timezone tz;
-	static GRand *rnd = NULL;
 
 	gettimeofday (&tv, &tz);
 
-	if (rnd == NULL)
-		rnd = g_rand_new_with_seed (tv.tv_usec ^ tv.tv_sec);
-	else
-		g_rand_set_seed (rnd, tv.tv_usec ^ tv.tv_sec);
-	randnums[0] += g_rand_int (rnd);
-	randnums[1] *= g_rand_int (rnd);
-	randnums[2] ^= g_rand_int (rnd);
+	/* the higher order bits of the seconds
+	   are quite uninteresting */
+	randnums[0] ^= next_rand_32 ((tv.tv_sec << 20) ^ tv.tv_usec);
 
-	randnums[3] += g_random_int ();
+	/* different method of combining */
+	randnums[1] ^= next_rand_32 (tv.tv_sec) ^ next_rand_32 (tv.tv_usec);
+
+	/* probably unneeded, but just being anal */
+	randnums[2] ^= (tv.tv_sec << 20) ^ tv.tv_usec;
+
+	/* probably unneeded, to guess above
+	   the number of invocation is likely needed
+	   anyway */
+	randnums[3] ++;
+
+	/* also hope that other places call
+	   g_random_int.  Note that on systems
+	   without /dev/urandom, this will yet again
+	   measure time the first time it's called and
+	   we'll add entropy based on the speed of the
+	   computer.  Yay */
 	randnums[4] ^= g_random_int ();
 }
 
@@ -116,8 +152,8 @@ data_seems_random (const char buf[], int size)
 		return FALSE;
 	for (i = 0; i < 10; i++) {
 		int idx = g_random_int_range (0, size);
-		if (i > 0 &&
-		    lastval != buf[idx])
+		if G_LIKELY (i > 0 &&
+			     lastval != buf[idx])
 			return TRUE;
 		lastval = buf[idx];
 	}
@@ -150,6 +186,9 @@ gdm_cookie_generate (GdmDisplay *d)
     /* use the last cookie */
     gdm_md5_update (&ctx, old_cookie, 16);
 
+    /* use some uninitialized stack space */
+    gdm_md5_update (&ctx, cookie, sizeof (cookie));
+
     pid = getppid();
     gdm_md5_update (&ctx, (unsigned char *) &pid, sizeof (pid));
     pid = getpid();
@@ -157,9 +196,9 @@ gdm_cookie_generate (GdmDisplay *d)
         
     for (i = 0; i < RNGS; i++) {
 	const char *file = rngs[i].path;
-	if (file == NULL)
+	if G_UNLIKELY (file == NULL)
 	    file = d->authfile;
-	if (file == NULL)
+	if G_UNLIKELY (file == NULL)
 	    continue;
 	fd = open (file, O_RDONLY|O_NONBLOCK
 #ifdef O_NOCTTY
@@ -169,7 +208,7 @@ gdm_cookie_generate (GdmDisplay *d)
 			|O_NOFOLLOW
 #endif
 		   );
-	if (fd >= 0) {
+	if G_LIKELY (fd >= 0) {
 	    /* Apparently this can sometimes block anyway even if it is O_NONBLOCK,
 	       so use select to figure out if there is something available */
 	    fd_set rfds;
@@ -182,21 +221,21 @@ gdm_cookie_generate (GdmDisplay *d)
 	    tv.tv_usec = 10*1000 /* 10 ms */;
 	    r = 0;
 
-	    if (rngs[i].seek > 0)
+	    if G_UNLIKELY (rngs[i].seek > 0)
 		lseek (fd, rngs[i].seek, SEEK_SET);
 
-	    if (select (fd+1, &rfds, NULL, NULL, &tv) > 0) {
+	    if G_LIKELY (select (fd+1, &rfds, NULL, NULL, &tv) > 0) {
 	        IGNORE_EINTR (r = read (fd, buf, MIN (sizeof (buf), rngs[i].length)));
 	    }
 
-	    if (r > 0)
+	    if G_LIKELY (r > 0)
 		gdm_md5_update (&ctx, buf, r);
 	    else
 		r = 0;
 
 	    IGNORE_EINTR (close (fd));
 
-	    if (r >= rngs[i].length &&
+	    if G_LIKELY (r >= rngs[i].length &&
 		data_seems_random (buf, r)) 
 		break;
 	}
