@@ -151,6 +151,7 @@ gchar *GdmSuspend = NULL;
 gchar *GdmSuspendReal = NULL;
 gchar *GdmServAuthDir = NULL;
 gchar *GdmUserAuthDir = NULL;
+gboolean GdmNeverPlaceCookiesOnNFS = TRUE;
 gchar *GdmUserAuthFile = NULL;
 gchar *GdmUserAuthFB = NULL;
 gchar *GdmPidFile = NULL;
@@ -350,6 +351,7 @@ gdm_config_parse (void)
     GdmSuspend = ve_config_get_string (cfg, GDM_KEY_SUSPEND);
     GdmUser = ve_config_get_string (cfg, GDM_KEY_USER);
     GdmUserAuthDir = ve_config_get_string (cfg, GDM_KEY_UAUTHDIR);
+    GdmNeverPlaceCookiesOnNFS = ve_config_get_bool (cfg, GDM_KEY_NEVERPLACECOOKIESONNFS);
     GdmUserAuthFile = ve_config_get_string (cfg, GDM_KEY_UAUTHFILE);
     GdmUserAuthFB = ve_config_get_string (cfg, GDM_KEY_UAUTHFB);
 
@@ -1701,6 +1703,7 @@ create_connections (void)
 					    gdm_handle_user_message,
 					    NULL /* data */,
 					    NULL /* destroy_notify */);
+		gdm_connection_set_nonblock (unixconn, TRUE);
 		gdm_connection_set_close_notify (unixconn,
 						 &unixconn,
 						 close_notify);
@@ -3217,6 +3220,12 @@ gdm_handle_user_message (GdmConnection *conn, const char *msg, gpointer data)
 {
 	gdm_debug ("Handling user message: '%s'", msg);
 
+	if (gdm_connection_get_message_count (conn) > 20) {
+		gdm_connection_write (conn, "ERROR 200 Too many messages\n");
+		gdm_connection_close (conn);
+		return;
+	}
+
 	if (strncmp (msg, GDM_SUP_AUTH_LOCAL " ",
 		     strlen (GDM_SUP_AUTH_LOCAL " ")) == 0) {
 		GSList *li;
