@@ -192,6 +192,8 @@ enum {
 #define GDM_KEY_RETRYDELAY "security/RetryDelay=3"
 #define GDM_KEY_DISALLOWTCP "security/DisallowTCP=true"
 
+#define GDM_KEY_NEVERPLACECOOKIESONNFS "security/NeverPlaceCookiesOnNFS=true"
+
 #define GDM_KEY_XDMCP "xdmcp/Enable=false"
 #define GDM_KEY_MAXPEND "xdmcp/MaxPending=4"
 #define GDM_KEY_MAXSESS "xdmcp/MaxSessions=16"
@@ -286,6 +288,7 @@ struct _GdmDisplay {
     GSList *local_auths; 
     gchar *userauth;
     gboolean authfb;
+    time_t last_auth_touch;
     gchar *command;
     gboolean failsafe_xserver;
     gboolean use_chooser; /* run chooser instead of greeter */
@@ -526,6 +529,9 @@ void		gdm_final_cleanup	(void);
  * is the gdm version and not a "protocol" revision, so you can't check
  * against a single version but check if the version is higher then some
  * value.
+ *
+ * You can only send a few commands at a time, so if you keep getting error
+ * 200 try opening a new socket for every command you send.
  */
 /* The user protocol, using /tmp/.gdm_socket */
 
@@ -535,6 +541,9 @@ void		gdm_final_cleanup	(void);
  * Arguments:  None
  * Answers:
  *   GDM <gdm version>
+ *   ERROR <err number> <english error description>
+ *      200 = Too many messages
+ *      999 = Unknown error
  */
 #define GDM_SUP_AUTH_LOCAL "AUTH_LOCAL" /* <xauth cookie> */
 /* AUTH_LOCAL: Setup this connection as authenticated for FLEXI_SERVER
@@ -551,6 +560,7 @@ void		gdm_final_cleanup	(void);
  *   ERROR <err number> <english error description>
  *      0 = Not implemented
  *      100 = Not authenticated
+ *      200 = Too many messages
  *      999 = Unknown error
  */
 #define GDM_SUP_FLEXI_XSERVER "FLEXI_XSERVER" /* <xserver type> */
@@ -569,6 +579,7 @@ void		gdm_final_cleanup	(void);
  *      4 = X too busy
  *      6 = No server binary
  *      100 = Not authenticated
+ *      200 = Too many messages
  *      999 = Unknown error
  */
 #define GDM_SUP_FLEXI_XNEST  "FLEXI_XNEST" /* <display> <uid> <xauth cookie> <xauth file> */
@@ -597,6 +608,7 @@ void		gdm_final_cleanup	(void);
  *      5 = Xnest can't connect
  *      6 = No server binary
  *      100 = Not authenticated
+ *      200 = Too many messages
  *      999 = Unknown error
  */
 #define GDM_SUP_CONSOLE_SERVERS  "CONSOLE_SERVERS" /* None */
@@ -614,6 +626,11 @@ void		gdm_final_cleanup	(void);
  *   for example).  If the display is an xnest display and is a console one
  *   (that is, it is an xnest inside another console display) it is listed
  *   and instead of vt, it lists the parent display in standard form.
+ *
+ *   ERROR <err number> <english error description>
+ *      1 = Not implemented
+ *      200 = Too many messages
+ *      999 = Unknown error
  */
 #define GDM_SUP_ALL_SERVERS  "ALL_SERVERS" /* None */
 /* ALL_SERVERS: List all servers, including console, remote, xnest.  This
@@ -628,6 +645,11 @@ void		gdm_final_cleanup	(void);
  *   <server> is <display>,<logged in user>
  *
  *   <logged in user> can be empty in case no one logged in yet
+ *
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      200 = Too many messages
+ *      999 = Unknown error
  */
 #define GDM_SUP_UPDATE_CONFIG "UPDATE_CONFIG" /* <key> */
 /* UPDATE_CONFIG: Tell the daemon to update config of some key.  Any user
@@ -667,6 +689,7 @@ void		gdm_final_cleanup	(void);
  *   ERROR <err number> <english error description>
  *      0 = Not implemented
  *      50 = Unsupported key
+ *      200 = Too many messages
  *      999 = Unknown error
  */
 #define GDM_SUP_GREETERPIDS  "GREETERPIDS" /* None */
@@ -676,6 +699,10 @@ void		gdm_final_cleanup	(void);
  * Arguments:  None
  * Answers:
  *   OK <pid>;<pid>;...
+ *   ERROR <err number> <english error description>
+ *      0 = Not implemented
+ *      200 = Too many messages
+ *      999 = Unknown error
  */
 #define GDM_SUP_CLOSE        "CLOSE" /* no arguments */
 /* CLOSE Answers: None
@@ -693,6 +720,23 @@ enum {
 		errno = 0;	\
 		expr;		\
 	} while G_UNLIKELY (errno == EINTR);
+
+#define NEVER_FAILS_seteuid(uid) \
+	{ int r = seteuid (uid); \
+	  if G_UNLIKELY (r != 0) \
+        gdm_fail ("GDM file %s: line %d (%s): Cannot run seteuid to %d", \
+		  __FILE__,						\
+		  __LINE__,						\
+		  __PRETTY_FUNCTION__,					\
+                  (int)uid);			}
+#define NEVER_FAILS_setegid(gid) \
+	{ int r = setegid (gid); \
+	  if G_UNLIKELY (r != 0) \
+        gdm_fail ("GDM file %s: line %d (%s): Cannot run setegid to %d", \
+		  __FILE__,						\
+		  __LINE__,						\
+		  __PRETTY_FUNCTION__,					\
+                  (int)gid);			}
 		
 
 #endif /* GDM_H */
