@@ -458,8 +458,6 @@ gdm_text_message_dialog (const char *msg)
 	msg_quoted = g_shell_quote (msg);
 	
 	dialog = g_find_program_in_path ("dialog");
-	if (dialog == NULL)
-		dialog = g_find_program_in_path ("gdialog");
 	if (dialog == NULL)
 		dialog = g_find_program_in_path ("whiptail");
 	if (dialog != NULL) {
@@ -527,8 +525,6 @@ gdm_text_yesno_dialog (const char *msg, gboolean *ret)
 	msg_quoted = g_shell_quote (msg);
 	
 	dialog = g_find_program_in_path ("dialog");
-	if (dialog == NULL)
-		dialog = g_find_program_in_path ("gdialog");
 	if (dialog == NULL)
 		dialog = g_find_program_in_path ("whiptail");
 	if (dialog != NULL) {
@@ -822,8 +818,8 @@ gdm_ensure_sanity (void)
 	old_euid = geteuid ();
 	old_egid = getegid ();
 
-	seteuid (0);
-	setegid (0);
+	NEVER_FAILS_seteuid (0);
+	NEVER_FAILS_setegid (0);
 
 	/* The /tmp/.ICE-unix check, note that we do
 	 * ignore errors, since it's not deadly to run
@@ -850,8 +846,8 @@ gdm_ensure_sanity (void)
 
 	umask (old_umask);
 
-	seteuid (old_euid);
-	setegid (old_egid);
+	NEVER_FAILS_seteuid (old_euid);
+	NEVER_FAILS_setegid (old_egid);
 }
 
 const GList *
