@@ -42,6 +42,18 @@ extern gboolean GdmAllowRoot;
 extern gboolean GdmAllowRemoteRoot;
 extern gint GdmRetryDelay;
 
+static char *selected_user = NULL;
+
+void
+gdm_verify_select_user (const char *user)
+{
+	g_free (selected_user);
+	if (ve_string_empty (user))
+		selected_user = NULL;
+	else
+		selected_user = g_strdup (user);
+}
+
 static void
 print_cant_auth_errbox (void)
 {
@@ -89,24 +101,40 @@ gdm_verify_user (GdmDisplay *d,
 		 const gchar *display,
 		 gboolean local) 
 {
-    gchar *login, *passwd, *ppasswd, *auth_errmsg;
+    gchar *login, *passwd, *ppasswd;
     struct passwd *pwent;
 
     if (local)
 	    gdm_slave_greeter_ctl_no_ret (GDM_STARTTIMER, "");
 
-    /* Ask for the user's login */
     if (username == NULL) {
+authenticate_again:
+	    /* Ask for the user's login */
+	    gdm_verify_select_user (NULL);
 	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, _("Please enter your username"));
 	    login = gdm_slave_greeter_ctl (GDM_PROMPT, _("Username:"));
 	    if (login == NULL ||
 		gdm_slave_greeter_check_interruption ()) {
-		    if (local)
-			    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
-		    g_free (login);
-		    return NULL;
+		    if ( ! ve_string_empty (selected_user)) {
+			    /* user selected */
+			    g_free (login);
+			    login = selected_user;
+			    selected_user = NULL;
+		    } else {
+			    /* some other interruption */
+			    if (local)
+				    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
+			    g_free (login);
+			    return NULL;
+		    }
 	    }
 	    gdm_slave_greeter_ctl_no_ret (GDM_MSG, "");
+
+	    if ( ! gdm_slave_check_user_wants_to_log_in (login)) {
+		    g_free (login);
+		    login = NULL;
+		    goto authenticate_again;
+	    }
     } else {
 	    login = g_strdup (username);
     }
@@ -139,7 +167,7 @@ gdm_verify_user (GdmDisplay *d,
 	    gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 
     if (pwent == NULL) {
-	    sleep (GdmRetryDelay);
+	    gdm_sleep_no_signal (GdmRetryDelay);
 	    gdm_error (_("Couldn't authenticate user \"%s\""), login);
 
 	    print_cant_auth_errbox ();
@@ -153,7 +181,7 @@ gdm_verify_user (GdmDisplay *d,
     /* Check whether password is valid */
     if (ppasswd == NULL || (ppasswd[0] != '\0' &&
 			    strcmp (crypt (passwd, ppasswd), ppasswd) != 0)) {
-	    sleep (GdmRetryDelay);
+	    gdm_sleep_no_signal (GdmRetryDelay);
 	    gdm_error (_("Couldn't authenticate user \"%s\""), login);
 
 	    print_cant_auth_errbox ();
